; 10.5.1 - Alarme Automotivo Simplificado - Verso 01

; Exemplo da pg. 213 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivo: Habituar o aluno ao uso de entradas e sadas binrias (0 e 1) em um sistema microcontrolado simplificado.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes do exemplo emulado no kit XM852:

; P3.5 --> Liga o alarme;
; P3.4 --> Desliga o alarme;
; P3.3 --> Boto de pnico;

; P1.7 --> Sensor de abertura da porta direita;
; P1.6 --> Sensor de abertura da porta esquerda;
; P1.5 --> Sensor de abertura da porta-malas;
; P1.4 --> Sensor de abertura do cap;

; P1.3 --> Sirene (Buzina) / Luz Panico (acionamento simultneo);
; P1.2 --> Led indicativo de acionamento do alarme;
; P1.1 --> Setas esquerda (frente e traseira simultneas);
; P1.0 --> Setas direita (frente e traseira simultneas).


org 00h 
ljmp programa_principal

org 13h 
ljmp dispara_alarme_panico  ; Interrupo externa INT1

org 30h	
programa_principal:
	mov IE,#10000100b ; Programao das Interrupes
	mov IP,#00000000b 		
	mov TCON,#00000101b 
	
	mov P1,#0F0h
 
inicio:
	jnb P3.5, liga_alarme ; Monitora as chaves do programa (acionamento e portas)
	jnb P3.4, inicio
	jb P1.4, porta_aberta
	jb P1.5, porta_aberta
	jb P1.6, porta_aberta
	jb P1.7, porta_aberta
	ljmp inicio          ; Programa principal em loop

; Incio do loop para ligar o alarme

liga_alarme:
	mov P1,#11111111b
	lcall tempo
	mov P1,#11110000b
loop:
	lcall tempo
	setb P1.2
	lcall tempo
	clr P1.2
	sjmp loop

; Incio da sequncia para desligar o alarme

desliga_alarme:
	mov P1,#11111111b
	lcall tempo_rapido
	mov P1,#11110000b
	lcall tempo_rapido
	mov P1,#11111111b
	lcall tempo_rapido
	mov P1,#11110000b
	ljmp inicio

; Incio do loop para contar o tempo
tempo:
	mov r2,#8
loop2:
	mov r1,#250
	jnb P3.4, desliga_alarme ; Desliga o alarme
	jb P1.4, dispara_alarme
	jb P1.5, dispara_alarme
	jb P1.6, dispara_alarme
	jb P1.7, dispara_alarme
loop1:
	mov r0,#250
	djnz r0,$
	djnz r1,loop1
	djnz r2,loop2
	ret

; Incio do loop para contar o tempo reduzido

tempo_rapido:
	mov r5,#8
loop3:
	mov r4,#100
loop4:
	mov r3,#100
	djnz r3,$
	djnz r4,loop4
	djnz r5,loop3
	ret

; Incio do loop para porta aberta

porta_aberta:
	jnb P3.5, dispara_alarme
	jb P1.4, porta_aberta
	jb P1.5, porta_aberta
	jb P1.6, porta_aberta
	jb P1.7, porta_aberta
	ljmp inicio

; Inicio do loop para disparar o alarme quando alguma porta for aberta

dispara_alarme:
	clr P1.2
	lcall tempo_rapido
	setb P1.0
	clr P1.1
	setb P1.3
	lcall tempo_rapido
	setb P1.1
	clr P1.0
	setb P1.3
	jnb P3.4, desliga_alarme ; Desliga o alarme
	ljmp dispara_alarme

; Incio da rotina para configurar o sinal de pnico

dispara_alarme_panico:
	clr P1.2
	lcall tempo_rapido
	setb P1.0
	clr P1.1
	setb P1.3
	lcall tempo_rapido
	setb P1.1
	clr P1.0
	setb P1.3
	jb P3.4, dispara_alarme_panico 
	jnb P3.4,$ 
	mov P1,#0F0h
        reti
end
