; 10.5.2 - Iluminacao Autonoma de Ambiente - Verso 01

; Exemplo da pg. 216 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivos: Complementar o exemplo anterior habituando o aluno ao uso de um display de 7 segmentos
; e apresentar uma soluo por mquina de estados.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes do exemplo emulado no kit XM852 e definio das variveis:

    LUZ equ P1.0    ; Aciona a Luz 
    SN1 equ P3.2    ; Sensor de entrada da porta 1      
    SN2 equ P3.3    ; Sensor de sada da porta 1             
    SN3 equ P3.4    ; Sensor de entrada da porta 2          
    SN4 equ P3.5    ; Sensor de sada da porta 2 
    QNT equ R0      ; Pessoas na sala  
    
; Display de 7 segmentos (contagem de at 9 pessoas - uso de um nico display)
    
                     
org 00h                       
   	 ljmp START   ; Inicia o programa principal
   	              
org 30h                  
START:                 
   	 mov P1, #00h
   	 mov   DPTR, #0FFC1h     ; Posio mapeada em memria do display de 7 segmentos  
   	 mov   A, #8               
   	 movx  @DPTR, A    
   	 clr   LUZ                
   	 mov   QNT, #0            
   	 mov   R1,  #0           
   	 mov   R2,  #0             
   	 lcall REFRESH_D7SEG      

; Varredura dos sensores

MAIN:                        
; Label das rotinas princiais
; Bloco de instrues que verifica qual sensor foi acionado primeiro
    	cjne R1, #0, END_1_0      ; C
    	jb SN1, SUBEND_1_0_0  
    	mov R1, #1       
SUBEND_1_0_0:         
	jb SN2, END_1_0       
	mov R1, #2       
END_1_0:
    ; [ESTADO 1.1]
    ; Bloco de instrues que verifica se o sensor 2 foi acionado depois do sensor 1]
    	cjne R1, #1, END_1_1     
        jb  SN2, SUBEND_1_1_0
        mov R1, #3       
        SUBEND_1_1_0:        
        jnb SN1, END_1_1     
        mov R1, #0        
END_1_1:
    ; [ESTADO 1.2]
    ; Bloco de instrues que verifica se o sensor 1 foi acionado depois do sensor 2]
        cjne R1, #2, END_1_2     
        jb  SN1, SUBEND_1_2_0 
        mov R1, #4       
SUBEND_1_2_0:         
        jnb SN2, END_1_2      
        mov R1, #0        
END_1_2:
    ; [ESTADO 1.3]
    ; Bloco de instrues que, sabendo que o sensor 1 e 2 foram acionados nessa ordem, espera eles serem desacionados nessa mesma ordem]
        cjne R1, #3, END_1_3     
        jnb SN2, END_1_3     
        mov R1, #1    
        jnb SN1, END_1_3  
        lcall INC_QNT 
        mov   R1, #0  
END_1_3:

    ; [ESTADO 1.4]
    ; Bloco de instrues que, sabendo que o sensor 2 e 1 foram acionados nessa ordem, espera eles serem desacionados nessa mesma ordem]
        cjne R1, #4, END_1_4      
        jnb SN1, END_1_4      
        mov R1, #2       
        jnb SN2, END_1_4 
        lcall DEC_QNT 
        mov   R1, #0  
END_1_4:
    ; [ESTADO 2.0]
    ; Bloco de instrues que verifica se o sensor 3 foi acionado primeiro]
        cjne R2, #0, END_2_0      
        jb SN3, SUBEND_2_0_0 
        mov R2, #1        
SUBEND_2_0_0:         
        jb SN4, END_2_0      
        mov R2, #2        
END_2_0:
    ; [ESTADO 1.1]
    ; Bloco de instrues que verifica se o sensor 4 foi acionado depois do sensor 3]
        cjne R2, #1, END_2_1     
        jb  SN4, SUBEND_2_1_0 
        mov R2, #3       
SUBEND_2_1_0:         
        jnb SN3, END_2_1      
        mov R2, #0       
END_2_1:
    ; [ESTADO 1.2]
    ; Bloco de instrues que verifica se o sensor 3 foi acionado depois do sensor 4]
        cjne R2, #2, END_2_2     
        jb  SN3, SUBEND_2_2_0 
        mov R2, #4       
SUBEND_2_2_0:         
        jnb SN4, END_2_2     
        mov R2, #0       
END_2_2:
    ; [ESTADO 1.3]
    ; Bloco de instrues que, sabendo que o sensor 3 e 2 foram acionados nessa ordem, espera eles serem desacionados nessa mesma ordem]
        cjne R2, #3, END_2_3      
        jnb SN4, END_2_3      
        mov R2, #1       
        jnb SN3, END_2_3 
        lcall INC_QNT 
        mov   R2, #0  
END_2_3:
    ; [ESTADO 1.4]
    ; Bloco de instrues que, sabendo que o sensor 4 e 1 foram acionados nessa ordem, espera eles serem desacionados nessa mesma ordem]
        cjne R2, #4, END_2_4      
        jnb SN3, END_2_4     
        mov R2, #2        
        jnb SN4, END_2_4  
        lcall DEC_QNT 
        mov   R2, #0  
END_2_4:
; Final do loop principal
        ljmp  MAIN     
; Subrotina que Incrementa a quantidade de pessoas contadas na sala
INC_QNT:
        cjne  QNT, #09h, INC1     
        lcall REFRESH_D7SEG  
        ret                   
INC1:                     
        inc   QNT             
        lcall REFRESH_D7SEG   
        setb  LUZ           
        ret                   
; Subrotina que DEcrementa a quantidade de pessoas contadas na sala
DEC_QNT:
        cjne  QNT, #00h, DEC1     
        lcall REFRESH_D7SEG  
        ret                  
DEC1:                   
        dec   QNT             
        lcall REFRESH_D7SEG  
        cjne  QNT, #00h, ENDS 
        clr LUZ           
ENDS:                     
        ret                   
; Subrotina que atualiza o display de sete segmentos
REFRESH_D7SEG:
    	mov   A, QNT              
    	mov   DPTR, #HEX_D7SEG    
    	movc  A, @A + DPTR       
    	mov   DPTR, #0FFC0h      
    	movx  @DPTR, A          
    	ret 
    	
; Tabela de converso hexadecimal para o display de 7 segmentos

HEX_D7SEG: db 3Fh, 06h, 5Bh, 4Fh, 66h, 6Dh, 7Dh, 07h, 7Fh, 67h, 77h, 7Ch, 39h, 5Eh, 79h, 71h
end                        