; 10.5.3 - Relgio Digital com Alarme - Verso 01

; Exemplo da pg. 219 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivos: Complementar os exemplos anteriores habituando o aluno ao uso de 4 displays de 7 segmentos (multiplexados)
; e ao clculo de temporizao;

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes do exemplo emulado no kit XM852:

; P3.5 --> Avana hora
; P3.4 --> Avana minuto
; P3.3 --> Boto de ajuste do alarme
; P3.2 --> Boto de ajuste do relgio

; P1.7 --> Boto para desligar o alarme

; P1.0 e P1.3 --> Alarme (representando o sinal a ser enviado a um Buzzer por. ex)
; P1.2 --> Led indicativo de ajuste do alarme
; P1.1 --> Led indicativo de ajuste da hora


org 00h
ljmp inicio
org 03h			; interrupo externa ext0 do relgio
ljmp ajuste_Relogio
org 0Bh			; interrupo do timer 0
ljmp att_Display
org 13h			; interrupo externa ext1 alarme
ljmp ajuste_Alarme
org 1Bh			; interrupo do timer 1
ljmp baseDeTempo

; ##################### AJUSTES INICIAIS #####################

org 30h
inicio:
	setb P3.2
	setb P3.3
	setb P3.4
	setb P3.5
	mov R0, #0h	
	mov R1, #0h	; registrador do display 1
	mov R2, #0h	; registrador do display 2
	mov R3, #2h	; registrador do display 3
	mov R4, #1h	; registrador do display 4
	mov R5, #0h	; auxiliar na contagem de minuto
	mov R6, #0h	; flag para ajuste alarme
	mov 24h, #0h	
	mov 25h, #0h	; POSIES DE MEMRIA DO ALARME
	mov 26h, #0h
	mov 27h, #0h
	mov P1, #0F0h	; chaves azuis descansam em nvel lgico alto e leds descansam em nvel lgico baixo
	mov SP, #2Fh	; troca da posio da pilha
	mov IE, #8FH	; habilita o timer 0, timer 1, iext 0 e iext 1
	mov IP, #0Ah	; Timer 0 > Timer 1 > Iext 0 > Iext 1
	mov TMOD, #11h	; Timers 0 e 1 no Modo 1
	mov TCON, #00h	; TR = 1, temporizador inicia em TH e TL // Interrupes ajustadas para acionar em nvel
	mov TL0, #69h 	; 150us de att dos displays
	mov TH0, #0FFh 	; 
	mov TL1, #3Fh	; programao do Timer 1 (base de tempo do relgio, calibrao da base de tempo, se necessrio)
	mov TH1, #4Ch	;
	setb TR0
	setb TR1
	
	sjmp $		; programa travado nesta linha enquanto nenhum timer ou interrupo externa for acionada
	
; ##################### LOOP QUE MANTM OS DISPLAYS ACESOS #####################
	
att_Display:

;	clr P1.0	LED DE DEPURAO
	clr TR0         ; programa o timer 
	mov TL0, #69h
	mov TH0, #0FFh
	setb TR0
	
	mov dptr, #0FFC1h ; Aponta para selecionar qual dos 4 displays ser acionado
	mov A, #08h     
	movx @dptr, A
	
	mov A, R1
	mov dptr, #tab    ; Aponta para tabela de converso hexadecimal para 7 segmentos
	movc A, @A+dptr
	mov dptr, #0FFC0h ; Aponta para carregar o valor a ser mostrado no display previamente selecionado
	movx @dptr, A
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop	
	mov A, #00h
	movx @dptr, A
	
	mov dptr, #0FFC1h
	mov A, #04h
	movx @dptr, A	
	mov A, R2
	mov dptr, #tab
	movc A, @A+dptr
	mov dptr, #0FFC0h
	movx @dptr, A
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop	
	mov A, #00h
	movx @dptr, A
	
	mov dptr, #0FFC1h
	mov A, #02h
	movx @dptr, A	
	mov A, R3
	mov dptr, #tab
	movc A, @A+dptr
	mov dptr, #0FFC0h
	movx @dptr, A
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	mov A, #00h
	movx @dptr, A
	
	mov dptr, #0FFC1h
	mov A, #01h
	movx @dptr, A	
	mov A, R4
	mov dptr, #tab
	movc A, @A+dptr
	mov dptr, #0FFC0h
	movx @dptr, A
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	mov A, #00h
	movx @dptr, A
	
	reti
	
; ##################### ROTINA DO TIMER 0 PARA AJUSTAR A BASE DE TEMPO DO RELGIO #####################
	
baseDeTempo: 		
;	setb P1.0		; LED DE DEPURAO
	clr TR1
	mov TL1, #3Fh		; programao do Timer 1 (base de tempo do relgio, calibrao da base de tempo, se necessrio)
	mov TH1, #4Ch		
	setb TR1
	
	cjne R5, #06h, nao_igual 
		mov R5, #0h
		lcall unidademin
		lcall comparaAlarme
		reti
	nao_igual:
	cjne R0, #0C8h, nao_igual_segundo
		inc R5
		mov R0, #0h
		reti
	nao_igual_segundo:
	inc R0
	reti 
	
; ##################### ROTINA PARA AJUSTAR O RELGIO (INTERRUPO EXTERNA 0) #####################
	
ajuste_Relogio:
	
	setb P1.1
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	clr P1.1
	
	jb P3.4, y
	jnb P3.4, $
	nop
	nop
	lcall unidademin
	
y:	jb P3.5, x
	jnb P3.5, $
	nop
	nop
	lcall unidadehora
	
x:	reti
	
; ##################### ROTINA PARA AJUSTAR O ALARME (INTERRUPO EXTERNA 1) #####################
	
ajuste_Alarme:	
	lcall guardaValor	

cont:	
	setb P1.2
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	clr P1.2
	
	jb P3.4, r
	jnb P3.4, $
	nop
	nop
	lcall unidademin
	mov 24h, R1
	mov 25h, R2
	
r:	jb P3.5, s
	jnb P3.5, $
	nop
	nop
	lcall unidadehora
	mov 26h, R3 
	mov 27h, R4

s:	mov R6, TCON
	cjne R6, #50h, cont
	lcall retomaValor
	reti

guardaValor:
					
	mov 20h, R1
	mov 21h, R2
	mov 22h, R3
	mov 23h, R4
	mov R1, 24h
	mov R2, 25h
	mov R3, 26h
	mov R4, 27h
	ret
	
retomaValor:

	mov R1, 20h
	mov R2, 21h
	mov R3, 22h
	mov R4, 23h
	ret

; ##################### SUB-ROTINAS DE COMPARAO PARA INCREMENTO DOS DISPLAYS #####################


unidademin:
	cjne R1, #09h, diferente9
		mov R1, #0h
		lcall dezenamin
		ret
	diferente9:
	inc R1
	ret
	
dezenamin:
	cjne R2, #05h, diferente5
		mov R2, #0h
		lcall unidadehora
		ret
	diferente5:
	inc R2
	ret
	
unidadehora:
	cjne R3, #09, dhora4
		mov R3, #0h
		lcall dezenahora
		ret
	dhora4:
		cjne R3, #03, dhora2		; se a unidade da hora for = 4, necessrio saber se a dezena da hora est em 2, se estiver, as horas voltam pra 00
			cjne R4, #02h, dhora2
			mov R3, #0h
			lcall dezenahora
			ret
		dhora2:
		inc R3				; se no estiver, adicionar uma hora
		ret
			
dezenahora:
	cjne R4, #02h, dhora5
		mov R4, #0h
		ret
	dhora5:
	inc R4
	ret
	
comparaAlarme:
	mov A, R1
	cjne A, 24h, nao_toca
	mov A, R2
	cjne A, 25h, nao_toca
	mov A, R3 
	cjne A, 26h, nao_toca
	mov A, R4
	cjne A, 27h, nao_toca
	
leds:	jnb P1.7, nao_toca
	setb P1.0
	clr P1.1
	clr P1.2
	setb P1.3
	
	lcall att_Display
	
	clr P1.0
	clr P1.1
	clr P1.2
	clr P1.3
	sjmp leds
	
	nao_toca:	
	ret
				
; ##################### TABELA DE DADOS PARA VARREDURA DO DISPLAY (0 A 9) #####################

; Converso de hexadecimal para 7 segmentos

tab: dB 3Fh,06h, 5Bh, 4Fh, 66h, 6Dh, 7Dh, 07h, 7Fh, 67h, 77h, 7Ch, 39h, 5Eh, 79h, 71h

end