; 10.5.4 - Maquina de Cafe Automatica Simples - Verso 01

; Exemplo da pg. 221 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivos: Complementar os exemplos anteriores habituando o aluno ao uso do LCD, Motor de Passo e 
; comunicao serial.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes do exemplo emulado no kit XM852:

; P1.7 = microchave - Aviso de colocao do copo na mquina;

; P1.0 = vlvula de gua;
; P1.1 = vlvula de caf em p;
; P1.2 = vlvula de leite em p;
; P1.3 = vlvula de chocolate em p;

; Motor de passo - Extrusora rotativa simples de uma rosca para empurrar os ingredientes em p at o copo;

; LCD - Informaes aos usurios;

; Teclado do PC - Escolha do tipo de bebida.


org 00h
LJMP inicializa

ORG 23h
LJMP serial ; interrupo serial

ORG 30h
inicializa:

MOV IE, #10010000b  ; programao da interrupo serial
MOV IP, #00h 				
MOV TCON, #00h 				
MOV SCON,#01010000b

; baud rate da serial			
MOV TMOD, #00100000b 			
MOV TH1, #0FDh	
setb TR1

mov A, #0				 
mov R1, #0				; usado para percorrer as tabelas com as frases enviadas ao LCD
mov R2, #0				; usado para salvar caractere a caractere as tabelas e compar-las com '.'
mov R3, #0				; usado para delay 
mov R4, #0				; Usado para receber caractere da serial (sbuf)
mov R5, #0				; Usado para delay
mov R6, #0				; Usado para monitorar a interrupo do preparo do caf
mov R7, #0				; usado para fazer a rotina do gira_motor 12 vezes (uma volta completa)


lcall instrucao4			
lcall para_tudo

inicio:
mov p1, #0F0h				

; Instrues para configurar LCD
lcall instrucao1
lcall instrucao2
lcall instrucao3
lcall instrucao4


; Pisca no LCD as escolhas
inicio_escolha_cafe:

lcall escreve_escolha_cafe
lcall linha2
lcall escreve_cafe_curto
lcall detecta_escolha
lcall linha3
lcall escreve_cafe_longo
lcall detecta_escolha
lcall linha4
lcall escreve_capuccino
lcall detecta_escolha
mov R2, #64
lcall delay
lcall instrucao4
lcall escreve_escolha_cafe                                                                                                  
lcall linha2
lcall escreve_cafe_leite
lcall detecta_escolha
lcall linha3
lcall escreve_cafe_chocolate
lcall detecta_escolha
mov R2, #64
lcall delay
lcall instrucao4
sjmp inicio_escolha_cafe


finalizacao:
lcall para_tudo
lcall instrucao4

finalizacao2:

JNB P1.7, pula_finalizacao
ljmp inicializa
pula_finalizacao:

JB P1.7, pula_finalizacao1
lcall linha1
lcall escreve_cafepronto
lcall linha3
lcall escreve_retirarcopo

pula_finalizacao1:

sjmp finalizacao2

serial:
mov R4, SBUF
clr RI
reti 

; sub-rotinas para enviar frases ao LCD
escreve_escolha_cafe:
mov A, R1
mov dptr, #escolhacafe
movc A, @A+dptr
lcall detecta_escolha
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #':', escreve_escolha_cafe
mov R1, #00h
ret

escreve_cafe_curto:
mov A, R1
mov dptr, #cafecurto
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_cafe_curto
mov R1, #00h
ret

escreve_cafe_longo:
mov A, R1
mov dptr, #cafelongo
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_cafe_longo
mov R1, #00h
ret

escreve_capuccino:
mov A, R1
mov dptr, #capuccino
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_capuccino
mov R1, #00h
ret

escreve_cafe_leite:
mov A, R1
mov dptr, #cafeleite
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_cafe_leite
mov R1, #00h
ret

escreve_cafe_chocolate:
mov A, R1
mov dptr, #cafechocolate
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_cafe_chocolate
mov R1, #00h
ret

escreve_inserir_copo:
mov A, R1
mov dptr, #inserircopo
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_inserir_copo
mov R1, #00h
ret

escreve_preparandocafe:
mov A, R1
mov dptr, #preparandocafe
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_preparandocafe
mov R1, #00h
ret

escreve_retirarcopo:
mov A, R1
mov dptr, #retirarcopo
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_retirarcopo
mov R1, #00h
ret

escreve_cafepronto:
mov A, R1
mov dptr, #cafepronto
movc A, @A+dptr
mov dptr, #0FFD2h
movx @dptr, A
mov R2, A
inc R1
lcall display_busy
cjne R2, #'.', escreve_cafepronto
mov R1, #00h
ret

; sub-rotinas para fazer as doses
verifica_cafe:
jb p1.7, pula_ver
mov R6, #1h

lcall instrucao4
;lcall linha1
lcall escreve_preparandocafe
lcall delay3
mov R7, #12
loop_v2: mov R5, #100
loop_v1: mov R3, #255
DJNZ R3, $
DJNZ R5, loop_v1

lcall gira_motor
setb P1.1
jnb p1.7, pula_ver2
cjne R6, #1, ver_comp
ljmp inicializa
ver_comp:
pula_ver2:
DJNZ R7, loop_v2
pula_ver:
ret

verifica_agua:
jb p1.7, pula_agua
mov R6, #1h

lcall instrucao4
;lcall linha1
lcall escreve_preparandocafe
lcall delay3
mov R7, #64
loop_agua2: mov R5, #125
loop_agua1: mov R3, #125
DJNZ R3, $
DJNZ R5, loop_agua1
setb P1.0
jnb p1.7, pula_agua2
cjne R6, #1, compara_agua
ljmp inicializa
compara_agua:
pula_agua2:
DJNZ R7, loop_agua2
pula_agua:
ret

verifica_leite:
jb p1.7, pula_leite
mov R6, #1h

lcall instrucao4
;lcall linha1
lcall escreve_preparandocafe
lcall delay3
mov R7, #12
loop_leite2: mov R5, #100
loop_leite1: mov R3, #255
DJNZ R3, $
DJNZ R5, loop_leite1
lcall gira_motor
setb P1.2
jnb p1.7, pula_leite2
cjne R6, #1, compara_leite
ljmp inicializa
compara_leite:
pula_leite2:
DJNZ R7, loop_leite2
pula_leite:
ret

verifica_chocolate:
jb p1.7, pula_chocolate

lcall instrucao4
;lcall linha1
lcall escreve_preparandocafe
lcall delay3
mov R6, #1h
mov R7, #12
loop_chocolate2: mov R5, #100
loop_chocolate1: mov R3, #255
DJNZ R3, $
DJNZ R5, loop_chocolate1
lcall gira_motor
setb P1.3
jnb p1.7, pula_chocolate2
cjne R6, #1, compara_chocolate
ljmp inicializa
compara_chocolate:
pula_chocolate2:
DJNZ R7, loop_chocolate2
pula_chocolate:
ret

; sub-rotina para fazer o preparo do caf escolhido
detecta_escolha:    
cjne R4, #'C', cafelongo1
lcall instrucao4
lcall linha1
lcall escreve_cafe_curto
lcall linha3
lcall escreve_inserir_copo
lcall linha4
voltacafecurto:
jb p1.7,cafelong
lcall verifica_cafe
lcall para_tudo
lcall verifica_agua
ljmp finalizacao
cafelong:
sjmp voltacafecurto


cafelongo1:
cjne R4, #'V', capuccino1
lcall instrucao4
lcall linha1
lcall escreve_cafe_longo
lcall linha3
lcall escreve_inserir_copo
lcall linha4
voltacafelongo1:
jb p1.7,ver_cafelongo1
lcall verifica_cafe
lcall verifica_cafe
lcall para_tudo
lcall verifica_agua
lcall verifica_agua
ljmp finalizacao
ver_cafelongo1:
sjmp voltacafelongo1


capuccino1:
cjne R4, #'B', cafeleite1
lcall instrucao4
lcall linha1
lcall escreve_capuccino
lcall linha3
lcall escreve_inserir_copo
lcall linha4
voltacapuccino1:
jb p1.7,ver_capuccino1
lcall verifica_cafe
lcall para_tudo
lcall verifica_leite
lcall verifica_leite
lcall para_tudo
lcall verifica_chocolate
lcall para_tudo
lcall verifica_agua
lcall verifica_agua
ljmp finalizacao
ver_capuccino1:
sjmp voltacapuccino1

cafeleite1:
cjne R4, #'N', leitechocolate1
lcall instrucao4
lcall linha1
lcall escreve_cafe_leite
lcall linha3
lcall escreve_inserir_copo
lcall linha4
voltacafeleite1:
jb p1.7,ver_cafeleite1

lcall verifica_cafe
lcall verifica_cafe
lcall para_tudo
lcall verifica_leite
lcall verifica_leite
lcall para_tudo
lcall verifica_agua
lcall verifica_agua

ljmp finalizacao
ver_cafeleite1:
sjmp voltacafeleite1

leitechocolate1:
cjne R4, #'M',fim_serial
lcall instrucao4
lcall linha1
lcall escreve_cafe_chocolate
lcall linha3
lcall escreve_inserir_copo
lcall linha4
voltaleitechocolate1:
jb p1.7,ver_leitechocolate1
lcall verifica_leite
lcall verifica_leite
lcall para_tudo
lcall verifica_chocolate
lcall verifica_chocolate
lcall para_tudo
lcall verifica_agua
lcall verifica_agua
ljmp finalizacao
ver_leitechocolate1:
sjmp voltaleitechocolate1

fim_serial:
ret

; sub-rotina para verificar se o LCD est apto para escrever frase
display_busy:
mov dptr, #0FFE2h
movx A, @dptr 
JB acc.7, display_busy
ret

; function set
instrucao1:
mov dptr, #0FFC2h
mov A, #00111000b ;38h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; display on/off control
instrucao2:
mov dptr, #0FFC2h
mov A, #00001110b ;0Eh
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; entry mode set (incrementa as posicoes no display)
instrucao3:
mov dptr, #0FFC2h
mov A, #00000110b ; 06h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; clear all no LCD
instrucao4:
mov dptr, #0FFC2h
mov A, #00000001b ;01h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; seleciona a primeira linha do LCD
linha1:
mov A, #80h
mov dptr, #0FFC2h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; seleciona a segunda linha do LCD
linha2:
mov A, #0h
mov dptr, #0FFC2h
mov A, #80h
add A, #40h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; seleciona a terceira linha do LCD
linha3:
mov A, #0h
mov dptr, #0FFC2h
mov A, #80h
add A, #10h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

; seleciona a quarta linha do LCD
linha4:
mov A, #0h
mov dptr, #0FFC2h
mov A, #80h
add A, #50h
movx @dptr, A
mov A,#00h
lcall display_busy
ret

delay:
;mov R2, #64
loop2: mov R1, #125
loop1: mov R0, #125
DJNZ R0, $
DJNZ R1, loop1
DJNZ R2, loop2
ret

delay2:
loop22: mov R1, #204
loop21: mov R0, #100
DJNZ R0, $
DJNZ R1, loop21
ret

delay3:
mov 52, #80
loop32: mov 51, #125
loop31: mov 50, #125
DJNZ 50, $
DJNZ 51, loop31
DJNZ 52, loop32
ret

; sub-rotina que faz o motor girar 30 graus
gira_motor:
mov dptr, #0FFC6h
mov A, #1h
movx @dptr, A
lcall delay2

mov dptr, #0FFC6h
mov A, #2h
movx @dptr, A
lcall delay2

mov dptr, #0FFC6h
mov A, #4h
movx @dptr, A
lcall delay2

mov dptr, #0FFC6h
mov A, #8h
movx @dptr, A
lcall delay2
mov dptr, #0FFC6h
mov A, #0h
movx @dptr, A
ret

; apaga os LEDs e desliga o motor
para_tudo:
clr P1.0
clr P1.1
clr P1.2
clr P1.3
mov dptr, #0FFC6h
mov A, #0h
movx @dptr, A
ret

; Lookup tables --> Frases parao LCD

escolhacafe: db 'ESCOLHA O CAFE:'
cafecurto: db 'C - CAFE CURTO.'
cafelongo: db 'V - CAFE LONGO.'
capuccino: db 'B - CAPUCCINO.'
cafeleite: db 'N - CAFE LEITE.'
cafechocolate: db 'M - LEITE CHOC.'
inserircopo: db 'INSERIR O COPO.'
preparandocafe: db 'PREPARANDO .'
cafepronto: db 'BEBIDA PRONTA.'
retirarcopo: db 'RETIRAR COPO.'

end
