; 10.5.5 - Cofre Digital de Hotel - Verso 01

; Exemplo da pg. 225 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivos: Complementar os exemplos anteriores habituando o aluno ao uso de senha (4 caracteres) via 
; comunicao serial.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes do exemplo emulado no kit XM852 e definio das variveis:

;PERIFRICOS:

;-CHAVE(SENSOR DE ABERTURA) - P1.7;

;-DISPLAY DE LCD;
;-MOTOR DE PASSO (TRAVA - FECHAMENTO E ABERTURA);
;-TECLADO(VIA SERIAL) - COLOCAO DA SENHA.


;CONTROLE DO MOTOR
MOTOR_CONTROL_BIT 	EQU 00H
MOTOR_TIME_COUNTER 	EQU 33H
MOTOR_PULSE_COUNTER	EQU 36h
MOTOR_SENTIDO	        EQU 04h
MOTOR_POSITION		EQU 38h
MOTOR_EXADDR		EQU  0FFC6h ; MAPEAMENTO EM MMORIA DO MOTOR DE PASSO

;CONTROLE DO LCD
LCD_CONTROL_BIT	 	EQU 01H
LCD_CLR_CMD		EQU 01H
LCD_MSG_CODE		EQU 35h
LCD_CLRSCR		EQU 05H
LCD_DATA_EXADDR		EQU 0FFD2H ; MAPEAMENTO EM MMORIA DO LCD - PROGRAMAO
LCD_CMD_EXADDR		EQU 0FFC2H
LCD_READ_EXADDR		EQU 0FFE2H
LCD_MESSAGE_CODE	EQU 3AH

;CONTROLE DE ACESSO - SENHA
;ARMAZENAMENTO DE SENHA
PW_SAVED	EQU 40h
PW_MASTER	EQU 48h
PW_BUFFER	EQU 3Fh
PW_COUNTER EQU 3Eh
PW_OK	EQU  08h
PW_ADRRESS EQU 39h

;CONTROLE DE LEITURA
;0 - READ;  ;1 - WRITE
PW_RW	EQU 03h
PW_ACTIVE	EQU 07h
PW_CORRECT 	EQU 09H
PW_ENABLE	EQU 0AH

DOOR_SENSOR 	EQU 02h
LCD_DATA_ADDR 	EQU 30h
ST_SW EQU 0Bh

;SAFE STATE:
;0 - INICIALIZAO
;1 - COFRE ABERTO
;2 - COFRE FECHADO 
;3 - COFRE TRANCADO
;4 - COFRE DESTRANCADO
;5 - SENHA INCORRETA - COFRE TRANCADO

SAFE_STATE	  EQU 37h
SAFE_STATE_CHANGE EQU 06h


ORG 00H
LJMP START_CONFIG

;SERIAL
ORG 23H 
LJMP SERIAL_COM

;TIMER
ORG 0BH
LJMP TIMER0_INT

ORG 30h
START_CONFIG:
	mov IE, #092h ; Programao das Interrupes
	mov IP, #00h
	
	mov TCON, #40h ; Programao dos Timers
	mov TMOD, #21h
	mov TL0, #22H
	mov TH0, #4Ch
	SETB TR0
	
	MOV P1,  #0F0h
	MOV TH1, #0FDH
	MOV TL1, #0FDH
	SETB TR1
	MOV SCON, #50h ; Programao da Serial
	
	;COFRE
	SETB DOOR_SENSOR
	MOV SAFE_STATE, #01
	SETB ST_SW
	
	;MOTOR
	CLR MOTOR_CONTROL_BIT
	SETB MOTOR_SENTIDO
	MOV MOTOR_POSITION, #08h
	MOV MOTOR_PULSE_COUNTER, #0
	
	;LCD
	CLR LCD_CONTROL_BIT
	MOV LCD_DATA_ADDR, #0
	MOV LCD_MESSAGE_CODE, #1
	
	;SENHA
	MOV PW_COUNTER, #0H
	SETB PW_RW
	SETB PW_OK
	CLR PW_ACTIVE
	CLR PW_ENABLE
	
INICIO:
	LCALL SAFE_CONTROL
	JNB MOTOR_CONTROL_BIT,  LCD_ON;
	LCALL MOTOR_CONTROL	
LCD_ON:	
	JNB LCD_CONTROL_BIT,  INICIO
	LCALL LCD_CONTROL
	LJMP INICIO

TIMER0_INT:
	PUSH ACC
	LCALL DOOR_READ
	SETB MOTOR_CONTROL_BIT	
TIMER_RETURN:
	POP ACC
	mov TL0, #22H
	mov TH0, #4CH
	RETI

DOOR_READ:
	SETB DOOR_SENSOR
CHAVE1: JB P1.7, CHANGE_ST_DOOR
	CLR DOOR_SENSOR
	
CHANGE_ST_DOOR:	
	MOV A, SAFE_STATE
	CJNE A, #1, ST2
	JNB DOOR_SENSOR, DOOR_RET
	MOV SAFE_STATE, #2
	SETB ST_SW
	JMP DOOR_RET
ST2:	CJNE A, #2,DOOR_RET
	JB DOOR_SENSOR, DOOR_RET
	MOV SAFE_STATE, #1
	SETB ST_SW
DOOR_RET:
	RET
	
SERIAL_COM:
	PUSH ACC
	MOV A, SAFE_STATE
	CJNE A, #05 , PRESS_ST_4
	MOV SAFE_STATE, #03
	LCALL ST_3_NOMOTOR
	SJMP SRET

 PRESS_ST_4:	
	CJNE A, #04, SAVE_CHAR
	SETB ST_SW
	MOV SAFE_STATE, #02
	SJMP SRET
SAVE_CHAR:	
	JNB PW_ENABLE,SRET
	MOV PW_BUFFER, SBUF
	LCALL PW_CONTROL
SRET:   POP ACC
	CLR RI
	RETI

SAFE_CONTROL:
			
	JB ST_SW, SAFE_VERIFY
	RET
SAFE_VERIFY:
	MOV A, SAFE_STATE	

	;ESTADO 0 - INICIALIZAO
	;LIMPAR LCD			
	CJNE A, #0,  SAFE_ST_1
	MOV  SAFE_STATE, #1
	SETB ST_SW
	JMP SAFE_CONTROL_RETURN
		
	;ESTADO 1 - COFRE ABERTO
	;MOSTRAR NA TELA A MENSAGEM 1
	;ESPERAR AT A PORTA SER FECHADA
	
SAFE_ST_1:
		CJNE A, #1, SAFE_ST_2
		;LCD
		SETB LCD_CLRSCR
		SETB LCD_CONTROL_BIT	
		MOV LCD_MESSAGE_CODE, #1
		MOV LCD_DATA_ADDR, #0
		
		CLR ST_SW		
		;SENHA
		MOV PW_COUNTER, #0H
		SETB PW_RW
		CLR PW_OK
		CLR PW_ACTIVE
		CLR PW_ENABLE
		
		JMP SAFE_CONTROL_RETURN
		
	;ESTADO 2 - COFRE FECHADO
	;RECEBER SENHA
	
SAFE_ST_2:		
		CJNE A, #2, SAFE_ST_3
		;LCD
		SETB LCD_CLRSCR
		SETB LCD_CONTROL_BIT
		MOV LCD_MESSAGE_CODE, #2
		MOV LCD_DATA_ADDR, #0
		;SENHA
		SETB PW_ENABLE
		MOV PW_COUNTER, #0H
		CLR PW_RW
		SETB PW_OK
		CLR PW_ACTIVE
		CLR ST_SW
		JMP SAFE_CONTROL_RETURN

	;ESTADO 3 - COFRE TRANCADO
SAFE_ST_3:		
		CJNE A, #3, SAFE_ST_4

		;MOTOR
		SETB MOTOR_CONTROL_BIT
		SETB MOTOR_SENTIDO
		MOV MOTOR_POSITION, #08h
		MOV MOTOR_PULSE_COUNTER, #24
		;LCD
ST_3_NOMOTOR:	SETB LCD_CLRSCR
		SETB LCD_CONTROL_BIT
		MOV LCD_MESSAGE_CODE, #3
		MOV LCD_DATA_ADDR, #0
		;SENHA
		SETB PW_ENABLE
		MOV PW_COUNTER, #0H
		SETB PW_RW
		SETB PW_OK
		SETB PW_ACTIVE
		CLR ST_SW		
		JMP SAFE_CONTROL_RETURN

		;ESTADO 4 - SENHA CORRETA - DESTRAVAR
SAFE_ST_4:		
		CJNE A, #4, SAFE_ST_5
		;LCD
		SETB LCD_CLRSCR
		SETB LCD_CONTROL_BIT
		MOV LCD_MESSAGE_CODE, #4
		MOV LCD_DATA_ADDR, #0
		;SENHA
		SETB PW_ENABLE
		MOV PW_COUNTER, #0H
		SETB PW_RW
		SETB PW_OK
		SETB PW_ACTIVE
		;MOTOR
		SETB MOTOR_CONTROL_BIT
		CLR MOTOR_SENTIDO
		MOV MOTOR_POSITION, #08h
		MOV MOTOR_PULSE_COUNTER, #24
		CLR ST_SW		
		JMP SAFE_CONTROL_RETURN	

		;ESTADO 5 - SENHA INCORRETA - DIGITE NOVAMENTE
SAFE_ST_5:		
		CJNE A, #5, SAFE_CONTROL_RETURN
		;LCD
		SETB LCD_CLRSCR
		SETB LCD_CONTROL_BIT
		MOV LCD_MESSAGE_CODE, #5
		MOV LCD_DATA_ADDR, #0
		;SENHA
		SETB PW_ENABLE
		MOV PW_COUNTER, #0H
		CLR PW_RW
		SETB PW_OK
		SETB PW_ACTIVE
		
		CLR ST_SW		
		JMP SAFE_CONTROL_RETURN		
			
SAFE_CONTROL_RETURN:		
		RET		
		
PW_CONTROL:
		;SE PW_RW = 0, LEITURA. 1 = ESCRITA
		MOV A, PW_COUNTER	
GO_WRITE:	JB PW_RW, SETPW		
		ADD A, #10h
SETPW:		ADD A, #PW_SAVED
		MOV R0, A
		MOV @R0, PW_BUFFER
		INC PW_COUNTER
		MOV A, PW_COUNTER
		CJNE A, #4,  PW_RETURN
		JB PW_ACTIVE, COMPARE
		CLR PW_RW
		SETB PW_ACTIVE
		MOV PW_COUNTER, #0
		CLR PW_OK
		MOV SAFE_STATE, #03h
		SETB ST_SW
		JMP PW_RETURN
		
RESETPW:
		SETB PW_OK
		MOV PW_COUNTER, #0
		SETB PW_ACTIVE
		CLR PW_RW
		JMP PW_RETURN
		
COMPARE:	;SE O DGITO EST CERTO, PW_OK = 1. EM CASO DE ERRO, = 0
		MOV A, SAFE_STATE
		CJNE A, #3, PW_RETURN
		MOV PW_COUNTER, #0
		MOV A, #PW_SAVED
		ADD A,#10H
		MOV R0, A		; R0 <-END SENHA DIGITADA
		MOV R1,#PW_SAVED	; R1 <-END SENHA SALVA
		
		MOV R3, #4 ; CONTADOR PARA COMPARAR OS ENDEREOS
CHARCMP:	MOV A, @R1
		MOV R2, A
		MOV A, @R0
		INC R0
		INC R1
		SUBB A, R2 
		JNZ WRONGPW
		DJNZ R3, CHARCMP
		JMP PW_RIGHT
WRONGPW:	CLR PW_OK 
		MOV SAFE_STATE, #05
		SETB ST_SW
		JMP PW_RETURN
PW_RIGHT:	MOV SAFE_STATE, #04
		SETB ST_SW
PW_RETURN:
		RET	
		
LCD_CONTROL:	
		;CODIGO DE MENSAGEM EM R0
		;SALVA O NUMERO DE CARACTERES ESCRITOS		
		;VERIFICA SE O LCD EST OCUPADO
		
		MOV DPTR, #LCD_READ_EXADDR
		MOVX A, @DPTR
		JB ACC.7 , LCD_RETURN
		MOV R0, LCD_MESSAGE_CODE
		
CLRSCR:		JNB LCD_CLRSCR, MSG0
		;LIMPAR O DISPLAY
		MOV DPTR, #LCD_CMD_EXADDR
		MOV A, #LCD_CLR_CMD
		MOVX @DPTR, A
		CLR LCD_CLRSCR
		JMP LCD_RETURN		
		
		;DETERMINAR O ENDEREO INICIAL DA MENSAGEM
MSG0:	CJNE R0, #0, MSG1
		MOV LCD_DATA_ADDR, #0
		JMP  LCD_NOMSG
		
MSG1:	CJNE R0, #1, MSG2
		MOV DPTR, #MENSAGEM1
		JMP MEMORY
		
MSG2:	CJNE R0, #2, MSG3
		MOV DPTR, #MENSAGEM2
		JMP MEMORY
		
MSG3:	CJNE R0, #3, MSG4
		MOV DPTR, #MENSAGEM3
		JMP MEMORY
		
MSG4:	CJNE R0, #4, MSG5
		MOV DPTR, #MENSAGEM4
		JMP MEMORY
		
MSG5:	CJNE R0, #5, LCD_RETURN
		MOV DPTR, #MENSAGEM5
		JMP MEMORY
		
MEMORY:		
		;VERIFICA CARACTER NA MEMRIA
		MOV  A, LCD_DATA_ADDR
		MOVC A, @A+DPTR
		INC LCD_DATA_ADDR
		;SE '$' -> LER O PROXIMO VALOR. SE REPETIR $, 
		;FIM DE MENSAGE. SENAO, LINHA NOVA
		CJNE A,#'$', LCD_SEND
		MOV A, LCD_DATA_ADDR
		MOVC A, @A+DPTR
		CJNE A, #'$', LCD_NEWLINE
		;TERMINA TRANSMISSAO
LCD_NOMSG:	CLR LCD_CONTROL_BIT
                JMP LCD_RETURN
LCD_NEWLINE:	
		ADD A, #80h
		MOV DPTR, #LCD_CMD_EXADDR
		MOVX @DPTR, A
		INC LCD_DATA_ADDR
		JMP LCD_RETURN						
LCD_SEND:
		;ENVIA CARACTER
		MOV DPTR, #LCD_DATA_EXADDR
		MOVX @DPTR, A		
LCD_RETURN:
		RET						

MOTOR_CONTROL:
		;COMO USAR:
		;SETAR A DIREO EM MOTOR_SENTIDO
		;VERIFICA SE EXISTEM PULSOS PARA O MOTOR
		;DESLIGA  SE NAO HOUVER PULSO
		MOV R7,  MOTOR_PULSE_COUNTER
		CJNE R7,  #0,  ROTATE
		MOV DPTR, #MOTOR_EXADDR
		MOV A, #0
		MOVX @DPTR,  A
		JMP MOTOR_RETURN
ROTATE:		
		;AJUSTA PARA A PRXIMA POSIO
		MOV A, MOTOR_POSITION		
		JB MOTOR_SENTIDO, CCW
		RR A				
		RR A
CCW:		RL A

		;CORRIGE OS ESTOUROS
		CJNE A, #80h, ESTOURO
		MOV A, #8h
		JMP PULSE
ESTOURO:	
		CJNE A, #10h,PULSE
		MOV A, #1h
		;ENVIA PARA O MOTOR
PULSE:
		MOV DPTR, #MOTOR_EXADDR
		MOVX @DPTR, A 
		MOV MOTOR_POSITION, A
		DEC MOTOR_PULSE_COUNTER		
		
 MOTOR_RETURN:
		CLR MOTOR_CONTROL_BIT
		RET
		
; LOCKUP TABLES

;MENSAGENS: $ = QUEBRA DE LINHA,  
;$$= FIM DE MENSAGEM

MENSAGEM1:
db "$", 0h, "COFRE ABERTO$" 
db 10h, "FAVOR FECHAR$"
db 50h, "A PORTA$$" 

MENSAGEM2:
db "$", 0h,"FAVOR PROGRAMAR$"
db 40h, "A NOVA SENHA$$"

MENSAGEM3:
db "$", 0h,"COFRE TRANCADO   $"
db 40h, "DIGITE A SENHA$$"

MENSAGEM5:
db "$", 0h,"SENHA INCORRETA$"
db 40h, "PRESSIONE $"
db 10h, "QUALQUER TECLA$$"

MENSAGEM4:
db "$", 0h,"COFRE DESTRAVADO$"
db 40h, "PRESSIONE $"
db 10h, "QUALQUER TECLA$$"

MENSAGEMX:
db "FAVOR REPETIR$"
db 40h,"A NOVA SENHA$$"

END
