; 10.6.1 - Furadeira Rotativa Automatizada - Verso 01

; Exemplo da pg. 228 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivos: Complementar os exemplos anteriores, adotando elementos j previamente utilizados, habituando assim
; o aluno em projetos bsicos de automao industrial emulados em um kit didtico.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes e definio das variveis do exemplo emulado no kit XM852:

; Motor de passo - Rotao do cilindro (O mandril, em nvel alto, fica a uma determinada distncia do cilindro na qual no
;  necessrio desligar o motor da furadeira para aguardar a rotao da pea. Assim, a rotao da pea j estar finalizada
; quando a broca encostar na pea iniciando a furao)

; LCD - Informaes aos usurios

; Teclado do PC - Senha (pr-definida) e habilitao (Para troca do nmero de furos, favor reiniciar)

SENSOR_PECA	equ P1.7 ; Avisa se pea inserida --> Sensor de presena de pea
SENSOR_ALTO	equ P1.6 ; Avisa quando curso da furadeira est no mnimo (Mandril totalmente levantado)
SENSOR_BAIXO	equ P1.5 ; Avisa quando curso da furadeira chega no mximo (Mandril totalmente abaixado)
LED1		equ P1.0 ; Representa Mandril abaixando
LED2		equ P1.1 ; Representa Mandril abaixando
LED3		equ P1.2 ; Representa motor da furadeira (sentido horrio)
LED4		equ P1.3 ; Representa motor da furadeira (sentido anti-horrio)

NUMERO_FUROS	equ 30h
FUROS_FEITOS	equ 31h
NUMERO_PECAS	equ 32h
NUMERO_PASSOS	equ 33h
SENHA_CERTA	equ 00h


org 00h                                         ;Incio do Programa
	sjmp inicio				;Pula as interrupes
org 03h
	ljmp INT0_INTERRUPT			;Interrupo da INT0, DESLIGANDO O SISTEMA POR SEGURANA
org 0Bh
	ljmp TIMER0_INTERRUPT			;Interrupo do TIMER0
org 30h

inicio:
	mov R1, #00h				
	mov R3, #00h
	mov NUMERO_PECAS, #00h			;Zera o nmero de peas
	mov P1, #0F0h				;Apaga os LEDs
	mov IE, #83h				;Configura as interrupes (EA = 1, ET0 = 1 e EX0 = 1)
	setb IT0				;Configura INT0 para acionamento por borda de descida
	mov TH1, #0FDh				;Configura o TIMER1 para gerar baud rate de 9600baud
	mov TL1, #0FDh				;Configura o TIMER1 para gerar baud rate de 9600baud
	mov TH0, #5Dh				;Configura o TIMER0 para gerar 24 passos por segundo
	mov TL0, #3Dh				;Configura o TIMER0 para gerar 24 passos por segundo
	mov TMOD, #21h				;Configura o TIMER1 no modo 2 e o TIMER0 no modo 1
	setb TR1				;Liga o TIMER1
	mov SCON, #40h				;Configura a serial no modo 1
	mov A, #38h				;Configura LCD: Modo 8 bits, 2 linhas e 5/7 pontos
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov A, #0Eh				;Configura LCD: Liga o display LCD e o cursor
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov A, #06h				;Configura LCD: Incrementa o cursor e no desloca o display
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	
loop_senha_errada:				;Loop para aguardar a senha correta
	mov DPTR, #MSG_SENHA1			;Coloca o DPTR em MSG_SENHA1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover	
	mov DPTR, #MSG_SENHA2			;Coloca o DPTR em MSG_SENHA2
	lcall LCD_FRASE				;Envia a frase
	mov A, #10h				;Move o cursor para a posio 10h
	lcall LCD_POSICAO			;Envia o comando para mover
	setb SENHA_CERTA			;Liga o bit SENHA_CERTA
	mov A, #00h
	
loop_verifica_senha:				;Loop para verificar se a senha est correta
	push ACC				;Envia o valor do acumulador para a pilha
	mov DPTR, #SENHA			;Coloca o DPTR em SENHA
	movc A, @A+DPTR				;L o valor de SENHA
	mov R0, A				;Move para o registrador R0 o valor do acumulador
	setb REN				;Ativa a recepo serial
	jnb RI, $				;Trava o programa enquanto no receber dados
	mov A, SBUF				;Move para o acumulador o valor recebido
	clr RI					;Limpa a flag de recepo de dados
	inc AR3
	cjne R3, #4, senha_continua2		;Verifica se foi pressionada a tecla Enter
	mov R3, #00h
	sjmp senha_fim				;Se foi pressionado Enter vai para senha_fim
	
senha_continua2:				;Caso no tenha sido pressionado Enter
	push ACC
	mov A, #2Ah
	lcall LCD_LETRA				;Envia a letra recebida para o display LCD
	pop ACC
	jnb SENHA_CERTA, senha_continua		;Se a flag SENHA_CERTA for 0 pula para senha_continua
	cjne A, AR0, senha_errada		;Compara a letra recebida com a letra no registrador R0
	sjmp senha_continua			;Se a letra for igual vai para senha_continua
	
senha_errada:					;Caso a seja diferente da letra armazenada
	clr SENHA_CERTA				;Limpa a flag SENHA_CERTA
	
senha_continua:					;Caso a flag SENHA_CERTA for 0 pula a comparao
	pop ACC					;Retira o valor do acumulador da pilha
	inc A					;Incrementa o valor do acumulador
	sjmp loop_verifica_senha		;Pula para loop_verifica_senha
	
senha_fim:					;Trmino da digitao da senha
	pop ACC					;Retira o valor do acumulador da pilha
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	jnb SENHA_CERTA, loop_senha_errada	;Se a flag SENHA_CERTA for 0 pula para loop_senha_errada
	
rec_num:					;Recebe o nmero de furos
	mov DPTR, #MSG_NUM_FUROS1		;Coloca o DPTR em MSG_NUM_FUROS1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_NUM_FUROS2		;Coloca o DPTR em MSG_NUM_FUROS2
	lcall LCD_FRASE				;Envia a frase
	mov A, #10h				;Move o cursor para a posio 10h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_NUM_FUROS3		;Coloca o DPTR em MSG_NUM_FUROS3
	lcall LCD_FRASE				;Envia a frase
	mov A, #50h				;Move o cursor para a posio 50h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_NUM_FUROS4		;Coloca o DPTR em MSG_NUM_FUROS4
	lcall LCD_FRASE				;Envia a frase
	
loop_rec_num:					;Loop para aguardar um nmero de furos vlido
	jnb RI, $				;Trava o programa enquanto no recebe dados na serial
	mov A, SBUF				;Coloca o dado recebido pela serial no acumulador
	clr RI					;Limpa a flag de recepo serial
	cjne A, #32h, compara_4			;Se no recebeu 2 testa 4
	sjmp confirmacao			;Se recebeu 2 vai para a confirmao
	
compara_4:					;Verifica se recebeu 4
	cjne A, #34h, compara_6			;Se no recebeu 4 testa 6
	sjmp confirmacao			;Se recebeu 4 vai para a confirmao
	
compara_6:					;Verifica se recebeu 6
	cjne A, #36h, compara_8			;Se no recebeu 6 testa 8
	sjmp confirmacao			;Se recebeu 6 vai para a confirmao
	
compara_8:					;Verifica se recebeu 8
	cjne A, #38h, loop_rec_num		;Se no recebeu 8 volta para aguardar um novo dado
	sjmp confirmacao			;Se recebeu 8 vai para a confirmao
	
confirmacao:					;Confirma se o nmero de furos est correto
	subb A, #30h				;Subtrai 30h do valor recebido para converter de ASCII para nmero
	mov NUMERO_FUROS, A			;Salva o nmero de furos na varivel NUMERO_FUROS
	mov FUROS_FEITOS, #00h			;Salva zero na varivel FUROS_FEITOS
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov A, NUMERO_FUROS			;Move para o acumulador o valor de NUMERO_FUROS
	add A, #30h				;Adiciona 30h para converter para ASCII
	lcall LCD_LETRA				;Envia o caractere
	mov A, #02h				;Move o cursor para a posio 02h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_CONFIRMACAO1		;Coloca o DPTR em MSG_CONFIRMACAO1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_CONFIRMACAO2		;Coloca o DPTR em MSG_CONFIRMACAO2
	lcall LCD_FRASE				;Envia a frase
	jnb RI,$				;Trava o programa enquanto no recebe dados na serial
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov A, SBUF				;Coloca o dado recebido pela serial no acumulador
	clr RI					;Limpa a flag de recepo serial
	cjne A, #53h, rec_num			;Verifica se recebeu 53h ('S'), se no retorna para escolha do nmero de furos 
	mov A, #240				;Move para o acumulador o valor 240
	mov B, NUMERO_FUROS			;Move para o registrador B o valor de NUMERO_FUROS
	div AB					;Divide A por B
	mov R0, A				;Move o valor do acumulador para o registrador R0
	mov B, #2				;Move para o registrador B o valor 2
	div AB					;Divide A por B
	add A, R0				;Adiciona ao valor do acumulador o valor do registrador R0
	mov B, #15				;Move para o registrador B o valor 15
	div AB					;Divide A por B
	mov B, #2				;Move para o registrador B o valor 2
	mul AB					;Multiplica A por B
	mov NUMERO_PASSOS, A			;Move para NUMERO_PASSOS o valor do acumulador
	
loop_verif_peca:				;Loop para verificar se a pea est inserida na mquina
	mov A, #01				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	jnb SENSOR_PECA, peca_inserida		;Se a pea estiver inserida vai para peca_inserida
	clr REN					;Desativa a recepo via serial
	mov DPTR, #MSG_INSERIR_PECA1		;Coloca o DPTR em MSG_INSERIR_PECA1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_INSERIR_PECA2		;Coloca o DPTR em MSG_INSERIR_PECA2
	lcall LCD_FRASE				;Envia a frase
	mov A, #10h				;Move o cursor para a posio 10h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_INSERIR_PECA3		;Coloca o DPTR em MSG_INSERIR_PECA3
	lcall LCD_FRASE				;Envia a frase
	jb SENSOR_PECA, $			;Trava o programa enquanto no houver pea na mquina
	sjmp loop_verif_peca			;Retorna para loop_verif_peca
	
peca_inserida:					;Caso a pea esteja inserida na mquina
	mov DPTR, #MSG_PECA_INSERIDA1		;Coloca o DPTR em MSG_PECA_INSERIDA1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_PECA_INSERIDA2		;Coloca o DPTR em MSG_PECA_INSERIDA2
	lcall LCD_FRASE				;Envia a frase
	mov A, #10h				;Move o cursor para a posio 10h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_PECA_INSERIDA3		;Coloca o DPTR em MSG_PECA_INSERIDA3
	lcall LCD_FRASE				;Envia a frase
	setb REN				;Ativa a recepo via serial
	
loop_iniciar:					;Loop para aguardar incio do processo
	jb SENSOR_PECA, loop_verif_peca		;Se no houver pea na mquina vai para loop_verif_peca
	jnb RI, loop_iniciar			;Se no recebeu dado via serial vai para loop_iniciar
	mov A, SBUF				;Coloca o dado recebido pela serial no acumulador
	clr RI					;Limpa a flag de recepo serial
	cjne A, #49h, loop_iniciar		;Caso no seja 49h ('I') retorna para loop_iniciar
	mov FUROS_FEITOS, #00h			;Move para FUROS_FEITOS o valor 00h
	
loop_processo:					;Loop do processo de furao
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov DPTR, #MSG_FURANDO1			;Coloca o DPTR em MSG_FURANDO1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_FURANDO2			;Coloca o DPTR em MSG_FURANDO2
	lcall LCD_FRASE				;Envia a frase
	mov A, NUMERO_FUROS			;Move para o acumulador o valor de NUMERO_FUROS
	clr C					;Limpa a flag de carry
	subb A, FUROS_FEITOS			;Subtrai do acumulador o valor de FUROS_FEITOS
	add A, #30h				;Adiciona 30h para converter para ASCII
	lcall LCD_LETRA				;Envia o valor de furos restantes
	setb LED3				;Liga o LED3 (Motor da furadeira - sentido horrio)
	clr LED4				;Desliga o LED4 (Motor da furadeira - sentido horrio)
	setb LED1				;Liga o LED1 (Atuador para abaixar a furadeira)
	jb SENSOR_BAIXO, $			;Trava o programa enquanto no ativar o sensor de posio baixo
	clr LED1				;Desliga o LED1 (Atuador para abaixar a furadeira)
	clr LED3				;Desliga o LED3 (Motor da furadeira - sentido anti-horrio)
	setb LED4				;Liga o LED4 (Motor da furadeira - sentido anti-horrio)
	setb LED2				;Liga o LED2 (Atuador para levantar a furadeira)
	jb SENSOR_ALTO, $			;Trava o programa enquanto no ativar o sensor de posio alto
	clr LED2				;Desliga o LED2 (Atuador para levantar a furadeira)
	clr LED3				;Desliga o LED3 (Motor da furadeira desligado)
	clr LED4				;Desliga o LED4 (Motor da furadeira desligado)
	mov A, FUROS_FEITOS			;Coloca o valor da varivel FUROS_FEITOS no acumulador
	inc A					;Incrementa o acumulador
	mov FUROS_FEITOS, A			;Coloca o valor do acumulador na varivel FUROS_FEITOS
	cjne A,NUMERO_FUROS, girar_peca		;Se no atingiu a quantidade de furos programada vai para girar_peca
	mov A, #01h				;Limpa o display LCD
	lcall LCD_COMANDO			;Envia o comando para o LCD
	mov DPTR, #MSG_FUROS_PRONTOS1		;Coloca o DPTR em MSG_FUROS_PRONTOS1
	lcall LCD_FRASE				;Envia a frase
	mov A, #40h				;Move o cursor para a posio 40h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_FUROS_PRONTOS2		;Coloca o DPTR em MSG_FUROS_PRONTOS2
	lcall LCD_FRASE				;Envia a frase
	mov A, #10h				;Move o cursor para a posio 10h
	lcall LCD_POSICAO			;Envia o comando para mover
	mov DPTR, #MSG_FUROS_PRONTOS3		;Coloca o DPTR em MSG_FUROS_PRONTOS3
	lcall LCD_FRASE				;Envia a frase
	mov A, NUMERO_PECAS			;Move para o acumulador o valor de NUMERO_PECAS
	inc A					;Incrementa o acumulador
	mov NUMERO_PECAS, A			;Move para NUMERO_PECAS o valor do acumulador
	add A, #30h				;Adiciona 30h para converter para ASCII
	lcall LCD_LETRA				;Envia o nmero de peas furadas
	jnb SENSOR_PECA, $			;Trava o programa para aguardar remoo da pea
	ljmp loop_verif_peca			;Retorna para loop_verif_peca
	
girar_peca:					;Gira a pea de acordo com o nmero de furos programado
	mov R2, NUMERO_PASSOS			;Move para o registrador R2 o valor de NUMERO_PASSOS
	setb TR0				;Liga o TIMER0
	sjmp loop_processo			;Retorna para loop_processo
	
LCD_COMANDO:					;Sub-rotina para enviar comandos para o display LCD
	lcall LCD_OCUPADO			;Aguarda o LCD ficar disponvel
	mov DPTR, #0FFC2h			;Coloca o DPTR em 0FFC2h (Endereo para gravao de instruo no LCD)
	movx @DPTR, A				;Envia para onde o DPTR aponta o valor do acumulador
	ret					;Retorno da sub-rotina
	
LCD_LETRA:					;Sub-rotina para enviar um caractere para o display LCD
	push DPH				;Envia o valor de DPH para a pilha
	push DPL				;Envia o valor de DPL para a pilha
	lcall LCD_OCUPADO			;Aguarda o LCD ficar disponvel
	mov DPTR, #0FFD2h			;Coloca o DPTR em 0FFD2h (Endereo para gravao de dados no LCD)
	movx @DPTR, A				;Envia para onde o DPTR aponta o valor do acumulador
	pop DPL					;Retira o valor de DPL da pilha
	pop DPH					;Retira o valor de DPH da pilha
	ret					;Retorno da sub-rotina
	
LCD_FRASE:					;Sub-rotina para enviar uma frase para o display LCD
	push DPH				;Envia o valor de DPH para a pilha
	push DPL				;Envia o valor de DPL para a pilha
	lcall LCD_OCUPADO			;Aguarda o LCD ficar disponvel
	pop DPL					;Retira o valor de DPL da pilha
	pop DPH					;Retira o valor de DPH da pilha
	mov A, #00h				;Move para o acumulador o valor 00h
	
LOOP_LETRA:					;Loop para enviar cada letra da frase
	push ACC				;Envia o valor de ACC para a pilha
	movc A, @A+DPTR				;L o valor de onde o DPTR aponta para o acumulador
	jz FIM_FRASE				;Se for 00h (marcador de fim da frase) pula para FIM_FRASE
	lcall LCD_LETRA				;Chama a sub-rotina de enviar um caractere
	pop ACC					;Retira o valor de ACC da pilha
	inc A					;Incrementa o acumulador
	sjmp LOOP_LETRA				;Pula para LOOP_LETRA
	
FIM_FRASE:					;Fim da frase
	pop ACC					;Retira o valor de ACC da pilha
	ret					;Retorno da sub-rotina
	
LCD_POSICAO:					;Sub-rotina para mover o cursor do display LCD
	add A, #80h				;Incrementa 80h no acumulador (comando para mover o cursor)
	lcall LCD_COMANDO			;Chama a sub-rotina de enviar comando para o LCD
	ret					;Retorno da sub-rotina

LCD_OCUPADO:					;Sub-rotina para verificar se o display LCD est ocupado
	push ACC				;Envia o valor de ACC para a pilha
	mov DPTR, #0FFE2h			;Coloca o DPTR em 0FFE2h (Endereo para leitura de instruo no LCD)
	
LOOP_VERIFICA:					;Loop para aguardar o display ficar disponvel
	movx A, @DPTR				;L o valor apontado por DPTR para o acumulador
	jb ACC.7, LOOP_VERIFICA			;Verifica pelo bit que indica se o LCD est ocupado
	pop ACC					;Retira o valor de ACC da pilha
	ret					;Retorno da sub-rotina
	
TIMER0_INTERRUPT:				;Interrupo do TIMER0
	push ACC				;Envia o valor de ACC para a pilha
	lcall GIRAR_MOTOR			;Chama a sub-rotina GIRAR_MOTOR
	mov A, R2				;Move para o acumulador o valor do registrador R2
	dec A					;Decrementa o acumulador
	mov R2, A				;Move para o registrador R2 o valor do acumulador
	clr TR0					;Desliga o TIMER0
	mov TH0, #5Dh				;Configura o TIMER0 para gerar 24 passos por segundo
	mov TL0, #3Dh				;Configura o TIMER0 para gerar 24 passos por segundo
	jz desliga_motor			;Caso tenha girado todos os passos pula para desliga_motor
	setb TR0				;Seno liga o TIMER0 novamente
	sjmp encerra_giro			;Pula para encerra_giro
	
desliga_motor:					;Desliga o motor de passo
	mov A, #00h				;Move para o acumulador o valor 00h
	mov DPTR, #0FFC6h			;Coloca o DPTR em 0FFC6h (Endereo do motor de passo)
	movx @DPTR, A				;Move para o endereo apontado por DPTR o valor do acumulador
	
encerra_giro:					;Encerra o giro
	pop ACC					;Retira o valor de ACC da pilha
	reti					;Retorno da interrupo
	
GIRAR_MOTOR:                                      ;Sub-rotina para girar o motor de passo	
	mov A, R1				;Move para o acumulador o valor do registrador R1
	mov DPTR, #PASSOS			;Coloca o DPTR em PASSOS
	movc A,@A+DPTR				;L o valor apontado por A+DPTR para o acumulador
	mov DPTR, #0FFC6h			;Coloca o DPTR em 0FFC6h (Endereo do motor de passo)
	movx @DPTR, A				;Move para o endereo apontado por DPTR o valor do acumulador
	mov A, R1				;Move para o acumulador o valor do registrador R1
	inc A					;Incrementa o acumulador
	cjne A, #4, girar_continua		;Verifica se o motor executou toda a sequncia de passos
	mov A, #00h				;Caso tenha executado retorna ao primeiro valor da sequncia
	
girar_continua:					;Caso no tenha executado continua a sequncia
	mov R1, A				;Move para o registrador R1 o valor do acumulador
	ret					;Retorno da sub-rotina
	
INT0_INTERRUPT:					;Interrupo INT0
	push ACC				;Envia o valor de ACC para a pilha
	mov A, P1				;Move para o acumulador o valor do port 1
	push ACC				;Envia o valor de ACC para a pilha
	anl A, #0F0h				;Filtra o valor das chaves do port 1
	mov P1, A				;Envia para o port 1 o valor do acumulador
	mov A, #00h				;Move para o acumulador o valor 00h
	mov DPTR, #0FFC6h			;Coloca o DPTR em 0FFC6h (Endereo do motor de passo)
	movx @DPTR, A				;Move para o endereo apontado por DPTR o valor do acumulador
	pop ACC					;Retira o valor de ACC da pilha
	mov C, TR0				;Move para C o valor de TR0
	mov ACC.4, C				;Move para ACC.4 o valor de C
	jb P3.3, $				;Trava o programa enquanto no pressionar o boto de reiniciar
	mov C, ACC.0				;Move para C o valor de ACC.0
	mov P1.0, C				;Move para P1.0 o valor de C
	mov C, ACC.1				;Move para C o valor de ACC.1
	mov P1.1, C				;Move para P1.1 o valor de C
	mov C, ACC.2				;Move para C o valor de ACC.2
	mov P1.2, C				;Move para P1.2 o valor de C
	mov C, ACC.3				;Move para C o valor de ACC.3
	mov P1.3, C				;Move para P1.3 o valor de C
	mov C, ACC.4				;Move para C o valor de ACC.4
	mov TR0, C				;Move para TR0 o valor de C
	pop ACC					;Retira o valor de ACC da pilha
	reti					;Retorno da interrupo	

PASSOS:			db 02h, 01h, 08h, 04h ; Passos do Motor de passo

; Lookup tables

SENHA:			db "1234", 0Dh ; Senha pr-programada (Alterar aqui a senha se necessrio)
MSG_SENHA1:		db "DIGITE A SENHA", 00h
MSG_SENHA2: 		db "(4 DIGITOS):",00h
MSG_NUM_FUROS1:		db "MAQ DE FURACAO", 00h
MSG_NUM_FUROS2:		db "DIGITE O NUMERO", 00h
MSG_NUM_FUROS3:		db "DE FUROS:", 00h
MSG_NUM_FUROS4:		db "2 4 6 OU 8", 00h
MSG_CONFIRMACAO1: 	db "FUROS", 00h
MSG_CONFIRMACAO2:	db "CONFIRMAR: S", 00h
MSG_INSERIR_PECA1:	db "MAQUINA LIVRE", 00h
MSG_INSERIR_PECA2:	db "FAVOR", 00h
MSG_INSERIR_PECA3:	db "INSERIR PECA", 00h
MSG_PECA_INSERIDA1:	db "PECA INSERIDA", 00h
MSG_PECA_INSERIDA2:	db "PARA INICIAR", 00h
MSG_PECA_INSERIDA3:	db "PRESSIONE I", 00h
MSG_FURANDO1:		db "FURANDO", 00h
MSG_FURANDO2:		db "RESTANDO: ", 00h
MSG_FUROS_PRONTOS1:	db "FUROS PRONTOS", 00h
MSG_FUROS_PRONTOS2:	db "FAVOR RETIRAR", 00h
MSG_FUROS_PRONTOS3:	db "A PECA ", 00h
end