; 10.6.2 - Dobradeira de Chapas Metalicas - Verso 01

; Exemplo da pg. 228 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivo: Complementar os exemplos anteriores, adotando elementos j previamente utilizados, habituando assim
; o aluno em projetos bsicos de automao industrial emulados em um kit didtico.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes e definio das variveis do exemplo emulado no kit XM852:

; Motor de passo - Movimenta a pea ("puxa" a pea);

; LCD - Informaes aos usurios;

; Teclado do PC - Habilitao das escolhas pelo usurio (via serial);

; P1.7 --> Sensor de presena de pea;
; P1.0 e P1.1 --> Atuador prende-solta pea;
; P1.2 e/ou P1.3 --> Atuadores prensa (desce-dobra-sobe).


ORG 00H               ;Incio do Programa
SJMP INICIO           ;Pula as reas de interrupes

ORG 03H
LJMP BOTAOSTOP

ORG 0BH
LJMP INTERNA0

ORG 23H
LJMP ISERIAL

ORG 33H

INICIO:

MOV IE, #93H         ;10010011 - PROGRAMA A INTERRUPO
MOV IP, #03H         ;PRIORIDADE DO BOTAO DE PARAR
MOV TMOD,  #21H      ;00100001 AJUSTA OS TIMER
MOV SCON, #50H       ;01010000 LIGA O SERIAL
MOV TH1, #0FDH       ;PROGRAMA OS TIMERS
MOV TL1, #0FDH
SETB TR1
CLR RI
MOV TH0, #21H 
MOV TL0, #7BH

LCALL PROGRAMALCD  ;ROTINA DE PROGRAMAO DO LCD

MOV P1, #0F0H
MOV R5, #1

LCALL ESCREVEF
SJMP $

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;PROGRAMACAO DO LCD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PROGRAMALCD:
MOV DPTR, #0FFC2H ; ENDEREO MAPEADO EM MEMRIA PARA PROGRAMAO DO LCD NO KIT
MOV A, #38H
MOVX @DPTR, A

LCALL CONFERE

MOV DPTR, #0FFC2H
MOV A, #0EH
MOVX @DPTR, A

LCALL CONFERE

MOV DPTR, #0FFC2H
MOV A, #06H
MOVX @DPTR, A

LCALL CONFERE

MOV DPTR, #0FFC2H
MOV A, #01H
MOVX @DPTR, A

RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;ESCREVENDO DO LCD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ESCREVEF: 
MOV A, #0
LOOPE: 
MOV DPTR, #TAB
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE
MOV A, #0C0H

LCALL TROCA

MOV A, #0
LOOPE2: 
MOV DPTR, #TAB1
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE2
MOV A, #90H

LCALL TROCA

MOV A, #0
LOOPE3: 
MOV DPTR, #TAB2
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE3
MOV A, #0D0H

LCALL TROCA

MOV A, #0
LOOPE4: 
MOV DPTR, #TAB3
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE4

SJMP FIMESCREVEF

TROCA:
MOV DPTR, #0FFC2H
MOVX @DPTR, A
LCALL CONFERE
RET

ESCREVE:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE
RET

ESCREVE2:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE2
RET

ESCREVE3:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE3
RET

ESCREVE4:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE4
RET

CONFERE:
MOV DPTR, #0FFE2H
LOOP: MOVX A, @DPTR
JB ACC.7, LOOP
RET

FIMESCREVEF:
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FIM  DO LCD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;ISERIAL;;;;;;;;;;;;;;;;;;;;;

ISERIAL:
MOV A, SBUF
CJNE A, #50, VERIFICA1
SUBB A, #48
MOV R7, A
MOV R6, #24
SJMP FINALVERIFICA

VERIFICA1:
CJNE A, #52, VERIFICA2
SUBB A, #48
MOV R7, A
MOV R6, #12
SJMP FINALVERIFICA

VERIFICA2:
CJNE A, #54, VERIFICA3
SUBB A, #48
MOV R7, A
MOV R6, #8
SJMP FINALVERIFICA

VERIFICA3:
CJNE A, #56, FINALOPE
SUBB A, #48
MOV R7, A
MOV R6, #6
SJMP FINALVERIFICA

FINALVERIFICA:
LCALL OPERACAO
FINALOPE:
CLR RI
RETI
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FIMISERIAL;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;ISTOP;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BOTAOSTOP:

PUSH ACC
PUSH 00H
PUSH 01H
PUSH 02H
PUSH 03H
PUSH 04H
PUSH 05H
PUSH 06H
PUSH 07H
PUSH DPH
PUSH DPL

MOV P1, #0F0H
MOV A, #00H

MOV DPTR, #0FFC6H
MOVX @DPTR, A

MOV DPTR, #0FFC2H
MOV A, #01H ;APAGAR O LCD
MOVX @DPTR, A

LCALL CONFERE

MOV R0, #0
LOOPTAB4:
MOV DPTR, #TAB4
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB4
SJMP FIMTAB4

ESCREVETAB4:
LCALL ESCREVELCD
SJMP LOOPTAB4

FIMTAB4:

JB P3.3, $
JNB P3.3, $

POP DPL
POP DPH
POP 07H
POP 06H
POP 05H
POP 04H
POP 03H
POP 02H
POP 01H
POP 00H
POP ACC


RETI
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FIMISTOP;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;OPERACAO;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
OPERACAO:

MOV DPTR, #0FFC2H
MOV A, #01H
MOVX @DPTR, A

LCALL CONFERE

MOV A, #0
LOOPE6: 
MOV DPTR, #TAB5
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE6
MOV A, #0C0H

LCALL TROCA

MOV A, #0
LOOPE7: 
MOV DPTR, #TAB6
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE7
MOV A, #90H

LCALL TROCA

MOV A, #0
LOOPE8: 
MOV DPTR, #TAB7
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE8

SJMP FIMLCDOP

ESCREVE6:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE6
RET

ESCREVE7:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE7
RET

ESCREVE8:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE8
RET

FIMLCDOP:
JB P1.7, $

MOV DPTR, #0FFC2H
MOV A, #01H
MOVX @DPTR, A

LCALL CONFERE

MOV A, #0
LOOPE9: 
MOV DPTR, #TAB8
MOV R0, A
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVE9

SJMP INICIARPRENSA

ESCREVE9:
MOV DPTR, #0FFD2H
MOVX @DPTR, A
LCALL CONFERE
MOV A, R0
INC A
LCALL LOOPE9
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;ANTES AJUSTE DO LCD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

INICIARPRENSA:


INC R6

MOV R4, #11H
MOV A, R6
MOV R3, A

SETB P1.0
SETB P1.1

LCALL DELAY

LOOPATUADOR:    

MOV A, R4
PUSH 05H
MOV R5, #1
SETB TR0
COMPARAMOTOR: CJNE R5, #0, COMPARAMOTOR
POP 05H
LCALL DOBRANDO
SETB P1.2 ;LIGA PRENSA
LCALL DELAY
CLR P1.2
SETB P1.3
LCALL DELAY
CLR P1.3
MOV A, R6
MOV R3, A
DJNZ R7, LOOPATUADOR

MOV R3, #50
MOV A, #88H
SETB TR0

ESPERANDO: CJNE R0, #15, ESPERANDO

LCALL PECADOBRADA
JNB P1.7, $

MOV DPTR, #0FFC2H
MOV A, #01H ;APAGAR O LCD
MOVX @DPTR, A

LCALL CONFERE

MOV R0, #0
LOOPTAB1:
MOV DPTR, #TAB1
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB1
SJMP FIMTAB1

ESCREVETAB1:
LCALL ESCREVELCD
SJMP LOOPTAB1

FIMTAB1:
MOV A, #0C0H
LCALL TROCA

MOV R0, #0
LOOPTAB2:
MOV DPTR, #TAB2
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB2
SJMP FIMTAB2

ESCREVETAB2:
LCALL ESCREVELCD
SJMP LOOPTAB2

FIMTAB2:
MOV A, #90H
LCALL TROCA

MOV R0, #0
LOOPTAB3:
MOV DPTR, #TAB3
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB3
SJMP FIMTAB3

ESCREVETAB3:
LCALL ESCREVELCD
SJMP LOOPTAB3

FIMTAB3:
INC R5

RET                ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FINAL DA ISERIAL;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DELAY:
PUSH 00H
PUSH 01H
PUSH 02H
MOV R2, #8
LOOPD1: MOV R1, #250
LOOPD: MOV R0, #250
DJNZ R0, $
DJNZ R1, LOOPD
DJNZ R2, LOOPD1
POP 02H
POP 01H
POP 00H
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;ESCREVER DURANTE OPERACAO LCD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DOBRANDO:

MOV DPTR, #0FFC2H
MOV A, #01H
MOVX @DPTR, A

LCALL CONFERE

MOV R0, #0
LOOPTAB9:
MOV DPTR, #TAB9
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB9
SJMP FIMTAB9

ESCREVETAB9:
LCALL ESCREVELCD
SJMP LOOPTAB9

FIMTAB9:
MOV A, #0C0H
LCALL TROCA


FIMDOBRANDO:
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FIMOPERACAO;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FINAL DA OPERACAO;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PECADOBRADA:

MOV A, #0C0H
LCALL TROCA

MOV DPTR, #0FFC2H
MOV A, #01H
MOVX @DPTR, A

LCALL CONFERE

CJNE R5, #1, TABELA11

MOV R0, #0
LOOPTAB10:
MOV DPTR, #TAB10
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB10
SJMP FIMTAB10

ESCREVETAB10:
LCALL ESCREVELCD
SJMP LOOPTAB10

FIMTAB10:
MOV A, #90H
LCALL TROCA

LJMP FIMDASTABELAS

TABELA11:

CJNE R5, #2, TABELA12

MOV R0, #0
LOOPTAB11:
MOV DPTR, #TAB11
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB11
SJMP FIMTAB11

ESCREVETAB11:
LCALL ESCREVELCD
SJMP LOOPTAB11

FIMTAB11:
MOV A, #90H
LCALL TROCA

SJMP FIMDASTABELAS

TABELA12:

CJNE R5, #3, TABELA13

MOV R0, #0
LOOPTAB12:
MOV DPTR, #TAB12
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB12
SJMP FIMTAB12

ESCREVETAB12:
LCALL ESCREVELCD
SJMP LOOPTAB12

FIMTAB12:
MOV A, #90H
LCALL TROCA

SJMP FIMDASTABELAS

TABELA13:

CJNE R5, #4, TABELA14

MOV R0, #0
LOOPTAB13:
MOV DPTR, #TAB13
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB13
SJMP FIMTAB13

ESCREVETAB13:
LCALL ESCREVELCD
SJMP LOOPTAB13

FIMTAB13:
MOV A, #90H
LCALL TROCA

SJMP FIMDASTABELAS

TABELA14:

CJNE R5, #5, TABELA15

MOV R0, #0
LOOPTAB14:
MOV DPTR, #TAB14
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB14
SJMP FIMTAB14

ESCREVETAB14:
LCALL ESCREVELCD
SJMP LOOPTAB14

FIMTAB14:
MOV A, #90H
LCALL TROCA

SJMP FIMDASTABELAS

TABELA15:

CJNE R5, #6, TABELA16

MOV R0, #0
LOOPTAB15:
MOV DPTR, #TAB15
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB15
SJMP FIMTAB15

ESCREVETAB15:
LCALL ESCREVELCD
SJMP LOOPTAB15

FIMTAB15:
MOV A, #90H
LCALL TROCA

SJMP FIMDASTABELAS

TABELA16:
MOV R5, #1

FIMDASTABELAS:

LCALL CONFERE

MOV R0, #0
LOOPTAB20:
MOV DPTR, #TAB20
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB20
SJMP FIMTAB20

ESCREVETAB20:
LCALL ESCREVELCD
SJMP LOOPTAB20

FIMTAB20:
MOV A, #0D0H
LCALL TROCA

MOV R0, #0
LOOPTAB21:
MOV DPTR, #TAB21
MOV A, R0
MOVC A, @A+DPTR
CJNE A, #'$', ESCREVETAB21
SJMP FIMTAB21

ESCREVETAB21:
LCALL ESCREVELCD
SJMP LOOPTAB21

FIMTAB21:
SJMP FIMDETUDO

ESCREVELCD:
MOV DPTR, #0FFD2H
MOVX @DPTR,A
PUSH ACC
LCALL CONFERE
POP ACC
INC R0
RET

FIMDETUDO:
RET

;;;;;;;;;;;;;;;;;;;;;;;;;;INTERNA0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

INTERNA0:
CJNE R7, #0, MOTOR
CLR TR0
MOV DPTR, #0FFC6H
MOVX @DPTR, A
DJNZ R3, FIMMOTORRETORNA
MOV DPTR, #0FFC6H
MOV A, #0
MOVX @DPTR, A
MOV R0, #15
SJMP FINALMOTOR

FIMMOTORRETORNA:
CLR P1.0
CLR P1.1
MOV TH0, #21H 
MOV TL0, #7BH 
RR A
SETB TR0
SJMP FINALMOTOR

MOTOR:
CLR TR0
MOV DPTR, #0FFC6H
MOVX @DPTR, A
DJNZ R3, FIMMOTOR
MOV DPTR, #0FFC6H
MOV R4, A
MOV A, #0
MOVX @DPTR, A
DEC R5
MOV TH0, #21H
MOV TL0, #7BH
SJMP FINALMOTOR

FIMMOTOR:
MOV TH0, #21H ;85
MOV TL0, #7BH ;ED
RL A
SETB TR0

FINALMOTOR:
RETI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;FIMINTERNA0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Lookup tables

TAB: DB '    PRENSA$'
TAB1: DB 'DIGITE O NUMERO$'
TAB2: DB '   DE DOBRAS$'
TAB3: DB '   2,4,6,8$'
TAB4: DB '   PARADO$'
TAB5: DB 'MAQUINA LIVRE$'
TAB6: DB 'FAVOR INSERIR$'
TAB7: DB 'PECA$'
TAB8: DB 'PECA INSERIDA$'
TAB9: DB 'DOBRANDO$'
TAB10: DB 'PECA DOBRADA 1$'
TAB11: DB 'PECA DOBRADA 2$'
TAB12: DB 'PECA DOBRADA 3$'
TAB13: DB 'PECA DOBRADA 4$'
TAB14: DB 'PECA DOBRADA 5$'
TAB15: DB 'PECA DOBRADA 6$'
TAB16: DB 'PECA DOBRADA 7$'
TAB17: DB 'PECA DOBRADA 8$'
TAB20: DB 'FAVOR RETIRAR$'
TAB21: DB 'A PECA$'

END

