; 10.6.3 - Maquina para Estiramento Plstico - Verso 01

; Exemplo da pg. 239 do Livro Tecnologias para Automao, 1Ed., Editora Blucher, 2024.

; Objetivo: Complementar os exemplos anteriores, adotando elementos j previamente utilizados, habituando assim
; o aluno em projetos bsicos de automao industrial emulados em um kit didtico.

; Programa desenvolvido para uso e com base nas caractersticas do KIT EXSTO XM852.
; Para uso em outras plataformas de desenvolvimento 8051, algumas caracteristicas necessitaro
; ser adaptadas, como, por ex., pinagem dos elementos de entrada e sada, mapeamento em memria, etc.

; O cdigo assembly bsico proposto a seguir poder ser utilizado, melhorado ou adaptado livremente.

; Representaes e definio das variveis do exemplo emulado no kit XM852:

; Motor de passo - Movimenta a pea (faz o estiramento);

; LCD - Informaes aos usurios;

; Teclado do PC - Habilitao das escolhas pelo usurio (Para troca do valor do estiramento, favor reiniciar);

; P1.4 --> Sensor de presena de pea;
; P1.0 e P1.1 --> Atuador da faca de corte;
; P1.2 e P1.3 --> Atuador de expulso da pea.


ORG 00h
	LJMP inicio
	
ORG 03h
	SJMP $ ; Trava simples de programa em caso de emergncia, para reiniciar aplicar reset.
	
ORG 23h
	LJMP serial

ORG 30h

inicio:
	
	MOV IE, #91h ; Programao das interrupes (EA+ES0)
	MOV SCON,#50h ; Serial modo 1
	MOV TMOD,#20h ; Timer1 modo 2
	
	;Programao do Timer1 para baud rate da comunicao serial
	MOV TH1,#0FDh
	MOV TL1,#0FDh
	SETb TR1
		
	;Leds
	MOV P1, #0F0H
	CLR P1.0 ; Faca acionada/Cortando
	CLR P1.1 ; Faca acionada/Cortando
	CLR P1.2 ; Expulsor de pea acionado/Expulsando Pea
	CLR P1.3 ; Expulsor de pea acionado/Expulsando Pea
		
	;Variveis
	MOV A,#00h
	MOV R0,#00h ; utilizado no delay
	MOV R1,#00h ; utilizado no delay
	MOV R2,#00h ; Seleo da Serial
	MOV R3,#00h ; utilizado no delay FULL
	MOV R4,#00h ; Tamanho do extiramento
	MOV R5,#00h ; Utilizado para girar as tabelas
	MOV R6,#00h ; Sentido de giro do motor de passo
	MOV R7,#00h ; Numero de peas extiradas
	
	;Inicializao do LCD
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#38h;primeira instruo de inicializao
	MOVX @dptr,A
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#0Eh;segunda instruo de inicializao
	MOVX @dptr,A
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#06h; terceira instruo de inicializao
	MOVX @dptr,A
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy
	;Fim da inicializao do lcd
	
;Programa principal:
	
	;Escrever:
	;MAQ DE EXTIRAR
	;DIGITE O TAMANHO DO
	;EXTIRAMENTO
	;1 2 3 OU 4 CM

	;Escrever MAQ DE EXTIRAR
	;Na linha 1
tabela1:
	MOV dptr,#tab1
	LCALL PegarPalavra
	CJNE A,#'!', cont0
	MOV R5,#00h
	SJMP cont1
cont0:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela1
cont1:
	;Escrever DIGITE O TAMANHO DO
	;Na linha 2
	MOV dptr,#0FFC2h
	MOV A,#0C0h
	MOVX @dptr,A
	LCALL display_busy
	
tabela2:
	MOV dptr,#tab2
	LCALL PegarPalavra
	CJNE A,#'!', cont2
	MOV R5,#00h
	SJMP cont3
cont2:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela2
cont3:
	;EXTIRAMENTO
	;Na linha 3
	MOV dptr,#0FFC2h
	MOV A,#90h
	MOVX @dptr,A
	LCALL display_busy
	
tabela3:
	MOV dptr,#tab3
	LCALL PegarPalavra
	CJNE A,#'!', cont4
	MOV R5,#00h
	SJMP cont5
cont4:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela3
cont5:
	;1 2 3 OU 4 CM
	;Na linha 4
	MOV dptr,#0FFC2h
	MOV A,#0D0h
	MOVX @dptr,A
	LCALL display_busy
tabela4:
	MOV dptr,#tab4
	LCALL PegarPalavra
	CJNE A,#'!', cont6
	MOV R5,#00h
	SJMP cont7
cont6:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela4
cont7:
;-----------------------------------------------------------------------
	;Espera selecionar o tamanho do extiramento
	MOV R2,#04h

naoselecionado:

	CJNE R2,#06h, naoselecionado
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy

;---------------------------------------------------------------------
checarpeca:
	;Verificao se possui pea ou no na mquina
	Jb P1.4,nope
	SJMP tempeca
	
;----------------------------------------------------------------------
nope:
	;Escrever
	;MAQUINA LIVRE
	;
	;FAVOR
	;INSERIR PECA
	
	;MAQUINA LIVRE
	;Na Linha 1
	MOV dptr,#0FFC2h
	MOV A,#80h
	MOVX @dptr,A
	LCALL display_busy
	
tabela5:
	MOV dptr,#tab5
	LCALL PegarPalavra
	CJNE A,#'!', cont8
	MOV R5,#00h
	SJMP cont9
cont8:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela5
cont9:

	;FAVOR
	;Na linha 3
	MOV dptr,#0FFC2h
	MOV A,#90h
	MOVX @dptr,A
	LCALL display_busy
	
tabela6:
	MOV dptr,#tab6
	LCALL PegarPalavra
	CJNE A,#'!', cont10
	MOV R5,#00h
	SJMP cont11
cont10:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela6
cont11:

	;INSERIR PEA
	;Na linha 4
	MOV dptr,#0FFC2h
	MOV A,#0D0h
	MOVX @dptr,A
	LCALL display_busy
	
tabela7:
	MOV dptr,#tab7
	LCALL PegarPalavra
	CJNE A,#'!', cont12
	MOV R5,#00h
	SJMP cont13
cont12:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela7
cont13:
	SJMP checarpeca

;-----------------------------------------------------------------------------------------------
	
;-------------------------------------------------------------------------------------------------
;Mquina com pea inserida
	
tempeca:

	;Escrever
	;PEA INSERIDA
	;
	;INICIAR PROCESSO
	
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy
	
	;PEA INSERIDA
	;Na linha 1
	
tabela8:
	MOV dptr,#tab8
	LCALL PegarPalavra
	CJNE A,#'!', cont14
	MOV R5,#00h
	SJMP cont15
cont14:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela8
cont15:
	;INICIAR PROCESSO
	;Na linha 3
	MOV dptr,#0FFC2h
	MOV A,#90h
	MOVX @dptr,A
	LCALL display_busy
	
tabela9:
	MOV dptr,#tab9
	LCALL PegarPalavra
	CJNE A,#'!', cont16
	MOV R5,#00h
	SJMP cont17
cont16:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela9
cont17:
;--------------------------------------------------------------------------------------
	;Espera a tecla I para iniciar o processo
	MOV R2,#05h

naoiniciar:
	CJNE R2,#09, naoiniciar
	
;--------------------------------------------------------------------------------------
;inicio do processo
	;Escrever
	;Extirando
	;Pea
	
	;Extirando
	;Na linha 1
	
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy
	
tabela10:
	MOV dptr,#tab10
	LCALL PegarPalavra
	CJNE A,#'!', cont18
	MOV R5,#00h
	SJMP cont19
cont18:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela10
cont19:
	
	;A Pea
	;Na linha 2
	
	MOV dptr,#0FFC2h
	MOV A,#0C0h
	MOVX @dptr,A
	LCALL display_busy
	
tabela11:
	MOV dptr,#tab11
	LCALL PegarPalavra
	CJNE A,#'!', cont20
	MOV R5,#00h
	SJMP cont21
cont20:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela11
cont21:
	
;1- rotacionar o motor de passo 1 volta no sentido anti-horrio(encostar na pea)
	MOV R6,#08h
	MOV R5,#48
loop20:	
	MOV dptr,#0FFC6h
	MOV A,R6
	MOVX @dptr,A
	LCALL delay50mS
	DEC R5
	RR A
	MOV R6,A
	CJNE R6,#80h,loop50
	MOV R6,#08h
loop50:
	CJNE R5,#00, loop20
	MOV A,#00h
	MOVX @dptr,A
	
	LCALL delay50mS
	LCALL delay50mS

;2-Atuador da faca de corte

	SETb P1.0	
	SETb P1.1
	LCALL delayFULL	
	CLR P1.0
	CLR P1.1	
	LCALL delayFULL
	
;3-Rotacionar o motor de passo o tamanho escolhido no sentido anti-horrio

	MOV R6,#08h
	MOV A,R4 ;valor do tamanho escolhido
	MOV R5,A
loop30:	
	MOV dptr,#0FFC6h
	MOV A,R6
	MOVX @dptr,A
	LCALL delay50mS
	DEC R5
	RR A
	MOV R6,A
	CJNE R6,#80h,loop60
	MOV R6,#08h
loop60:
	CJNE R5,#00, loop30
	MOV A,#00h
	MOVX @dptr,A
	
	LCALL delayFULL
	
;4-Retorno do motor de passo pra posio original:

	MOV R6,#01h
	ADD A,#48
	ADD A,R4
	MOV R5,A
loop21:	
	MOV dptr,#0FFC6h
	MOV A,R6
	MOVX @dptr,A
	LCALL delay50mS
	DEC R5
	RL A
	MOV R6,A
	CJNE R6,#10h,loop51
	MOV R6,#01h
loop51:
	CJNE R5,#00, loop21
	MOV A,#00h
	MOVX @dptr,A
	
	LCALL delay50mS
	LCALL delay50mS	
	
;5-Expulso da pea
	
	SETb P1.2
	SETb P1.3
	
	LCALL delayFULL
	
	CLR P1.2
	CLR P1.3
	
	LCALL delayFULL
	
	INC R7

	
;Fim do processo de extiramento
;--------------------------------------------------------------
	;Escrever:
	;PEA EXTIRADA (N)
	;
	;FAVOR RETIRAR
	;A PEA
	
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy

	;Escrever
	;PEA EXTIRADA (N)
	;Na linha 1
	
tabela12:
	MOV dptr,#tab12
	LCALL PegarPalavra
	CJNE A,#'!', cont22
	MOV R5,#00h
	SJMP cont23
cont22:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela12
cont23:
	;N- numero de peas extiradas
	MOV dptr,#0FFD2h
	MOV A,R7
	ADD A,#30h
	MOVX @dptr,A
	LCALL display_busy
	
	;Escrever:
	;Favor Retirar
	;Na linha 3
	
	MOV dptr,#0FFC2h
	MOV A,#90h
	MOVX @dptr,A
	LCALL display_busy
	
tabela13:
	MOV dptr,#tab13
	LCALL PegarPalavra
	CJNE A,#'!', cont24
	MOV R5,#00h
	SJMP cont25
cont24:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela13
cont25:

	;Escrever:
	;A pea
	;Na linha 4
	
	MOV dptr,#0FFC2h
	MOV A,#0D0h
	MOVX @dptr,A
	LCALL display_busy
	
tabela14:
	MOV dptr,#tab14
	LCALL PegarPalavra
	CJNE A,#'!', cont26
	MOV R5,#00h
	SJMP cont27
cont26:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela14
cont27:
;------------------------------------------------
;Verifica se retirou a pea

nope2:
	Jnb P1.4,nope2
	
	LCALL delay50mS
	
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy
	
	LJMP checarpeca	
	
;--------------------------------------------------------------------------------------
;Emergencia/Reset e Reset

emergencia:
	
	CLR RI	
	POP 28h
	POP 29h
	
	;Reset das variveis
	;Motor de passo
	MOV A,#00h
	MOVX @dptr,A
	
	LCALL delay50mS
	
	;Subir a faca
	CLR P1.0
	CLR P1.1
	LCALL delay50mS
	
	;Recuar pisto
	CLR P1.2
	CLR P1.3
	
	LCALL delay50mS
	
;escrever EMERGENCIA
	
	LCALL display_busy
	MOV dptr,#0FFC2h
	MOV A,#01h;quarta instruo de inicializao, clean clr e por cursor no 00
	MOVX @dptr,A
	LCALL display_busy
	
	MOV R5, #00h
	
tabela15:
	MOV dptr,#tab15
	LCALL PegarPalavra
	CJNE A,#'!', cont28
	MOV R5,#00h
	SJMP cont29
	
cont28:
	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela15
	
cont29:

; ESCREVER RETIRAR A PEA NA 3 LINHA DO LCD

	MOV dptr,#0FFC2h
	MOV A,#90h
	MOVX @dptr,A
	LCALL display_busy
	
tabela16:

	MOV dptr,#tab16
	LCALL PegarPalavra
	CJNE A,#'!', cont30
	MOV R5,#00h
	SJMP cont31
	
cont30:

	MOV dptr,#0FFD2h
	MOVX @dptr,A
	LCALL display_busy
	SJMP tabela16
	
cont31:

;verificar se a pea foi retirada
	JNB P1.4, $		
	PUSH 00h
	PUSH 00h	
	RETI
	
; RESET - s habilitado entre os processos	
resetar:
	CLR RI	
	POP 28h
	POP 29h	
	PUSH 00h
	PUSH 00h

	;Reset das variveis
	;Motor de passo
	MOV A,#00h
	MOVX @dptr,A
	
	RETI	

;Subrotinas:
;---------------------------------------------------------------------------------------------

display_busy:
	MOV dptr,#0FFE2h
	loop:
		MOVX A,@dptr
		Jb Acc.7, loop
	RET
	
PegarPalavra:
	MOV A,R5
	MOVC A,@A+dptr
	INC R5
	RET
	
delay50mS:
	MOV R1,#100
loop75:
	MOV R0,#250
	DJNZ R0,$
	DJNZ R1, loop75
	RET
	
delayFULL:
	MOV R3,#20
loop78:
	MOV R1,#250
loop76:
	MOV R0,#250
	DJNZ R0,$
	DJNZ R1, loop76
	DJNZ R3, loop78
	RET
	
;Interrupes:
;-----------------------------------------------------------------------------------------------------
	
serial:
	MOV A,SBUF ; valor escrito na serial pelo PC
	
	CJNE R2,#04, jp2
	;Tamanho dos extiramentos
	;Tam1
	CJNE A,#'1', tam2
	MOV R4,#12
	MOV R2,#06h
tam2:
	CJNE A,#'2', tam3
	MOV R4,#24
	MOV R2,#06h
	
tam3:
	CJNE A,#'3', tam4
	MOV R4,#36
	MOV R2,#06h
	
tam4:
	CJNE A,#'4', FIM8
	MOV R4,#48
	MOV R2,#06h	
	
jp2:
	CJNE R2, #05, FIM8
	;Iniciar Processo
	CJNE A,#'I', FIM5
	MOV R2,#09h
	SJMP FIM6
	
FIM5:	
	; RESETAR
	CJNE A, #'R', FIM8
	MOV R2, #09h
	LJMP resetar
	
FIM8:
	
	CJNE A,#'E',FIM6
	LJMP emergencia
	
FIM6:
	CLR RI
	RETI
	
	
; Look up tables
;------------------------------------------------------------------------------------------------------

tab1: db 'MAQ DE EXTIRAR!'

tab2: db 'DIGITE O TAMANHO!'

tab3: db 'DO EXTIRAMENTO!'

tab4: db '1 2 3 OU 4 CM!' 

tab5: db 'MAQUINA LIVRE!'

tab6: db 'FAVOR!'

tab7: db 'INSERIR PECA!'

tab8: db 'PECA INSERIDA!'

tab9: db 'INICIAR PROCESSO!'

tab10: db 'EXTIRANDO!'

tab11: db 'A PECA!'

tab12: db 'PECA EXTIRADA !'

tab13: db 'FAVOR RETIRAR!'

tab14: db 'A PECA!'

tab15: db 'EMERGENCIA!'

tab16: db 'RETIRAR A PECA!'

END
	